/* $Id$

This file is part of libmspcore
Copyright © 2006-2008, 2011  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#ifndef MSP_CORE_APPLICATION_H_
#define MSP_CORE_APPLICATION_H_

namespace Msp {

/**
Base class for applications.  Inherit the main class from this and add a static
member of type RegApp<MainClass>.
*/
class Application
{
protected:
	class Starter
	{
	protected:
		Starter();
	public:
		virtual ~Starter() { }

		virtual Application *create_app(int, char **) = 0;
		virtual void usage(const char *, const char *, bool) = 0;
	};

	bool done;
	int exit_code;

private:
	static Starter *starter_;
	static Application *app_;
	static void *data_;

protected:
	Application();
public:
	virtual ~Application() { }

	static int run(int, char **, void * =0);
	static void usage(const char *, const char *, bool);
	static void *get_data() { return data_; }

protected:
	virtual int main();
	void catch_signal(int);
	void exit(int);
	virtual void tick() { }
	virtual void sighandler(int) { }
private:
	static void sighandler_(int);
};

	Application(const Application &);
	Application &operator=(const Application &);

template<typename T>
class RegisteredApplication: public Application
{
private:
	class Starter: public Application::Starter
	{
	public:
		Application *create_app(int argc, char **argv) { return new T(argc, argv); }
		void usage(const char *r, const char *a, bool b) { T::usage(r, a, b); }
	};

	static Starter starter_;

protected:
	// Force the starter into existence
	RegisteredApplication() { (void)starter_; }
};

template<typename T>
typename RegisteredApplication<T>::Starter RegisteredApplication<T>::starter_;

} // namespace Msp

#endif
